@@ -26,6 +26,9 @@ import yaml
 from header2whatever.config import Config
 from header2whatever.parse import ConfigProcessor
 
+from urllib.error import HTTPError
+import dataclasses
+
 from setuptools import Extension
 
 from .devcfg import get_dev_config
@@ -38,7 +41,7 @@ from .download import download_maven
 
 class Wrapper:
     """
-        Wraps downloading bindings and generating them
+    Wraps downloading bindings and generating them
     """
 
     # Used during preprocessing
@@ -111,6 +114,9 @@ class Wrapper:
             # Add self to extension so that build_ext can query it on OSX
             self.extension.rpybuild_wrapper = self
 
+            # Used if the maven download fails
+            self.supported_platforms = setup.project.supported_platforms
+
         if self.cfg.generate and not self.cfg.generation_data:
             raise ValueError(
                 "generation_data must be specified when generate is specified"
@@ -129,7 +135,47 @@ class Wrapper:
         self.dev_config = get_dev_config(self.name)
 
     def _extract_zip_to(self, classifier, dst, cache):
-        download_maven(self.cfg.maven_lib_download, classifier, dst, cache)
+        try:
+            download_maven(self.cfg.maven_lib_download, classifier, dst, cache)
+        except HTTPError as e:
+            # Check for a 404 error and raise an error if the platform isn't supported.
+            if e.code != 404:
+                raise e
+            else:
+                platform_dict = dataclasses.asdict(self.platform)
+
+                os = platform_dict["os"]
+                arch = platform_dict["arch"]
+
+                is_os_supported = False
+                is_arch_supported = False
+
+                for supp_plat in self.supported_platforms:
+                    if supp_plat.os is None or supp_plat.os == os:
+                        is_os_supported = True
+                        if supp_plat.arch is None or supp_plat.arch == arch:
+                            is_arch_supported = True
+
+                if not (is_os_supported and is_arch_supported):
+                    if arch == "x86":
+                        arch = "32-bit"
+                    elif arch == "x86-64":
+                        arch = "64-bit"
+
+                    if os == "osx":
+                        os = "macOS"
+
+                    if not is_os_supported:
+                        arch = ""
+
+                    msg_plat = "{}{}{}".format(arch, " " if arch != "" else "", os)
+
+                    err_msg = "{} is not supported on {}!".format(
+                        self.pypi_package, msg_plat
+                    )
+
+                    raise OSError(err_msg)
+                raise e
 
     def _add_generated_file(self, fullpath):
         if not isdir(fullpath):
@@ -659,7 +705,8 @@ class Wrapper:
     def _write_wrapper_hpp(self, outdir, classdeps):
 
         decls = []
-        calls = []
+        begin_calls = []
+        finish_calls = []
 
         def _clean(n):
             tmpl_idx = n.find("<")
@@ -701,8 +748,10 @@ class Wrapper:
         ordering.extend(toposort.toposort_flatten(to_sort, sort=True))
 
         for name in ordering:
-            decls.append(f"void init_{name}(py::module &m);")
-            calls.append(f"    init_{name}(m);")
+            decls.append(f"void begin_init_{name}(py::module &m);")
+            decls.append(f"void finish_init_{name}();")
+            begin_calls.append(f"    begin_init_{name}(m);")
+            finish_calls.append(f"    finish_init_{name}();")
 
         content = (
             inspect.cleandoc(
@@ -716,13 +765,16 @@ class Wrapper:
         ##DECLS##
 
         static void initWrapper(py::module &m) {
-        ##CALLS##
+        ##BEGIN_CALLS##
+
+        ##FINISH_CALLS##
         }
         
         """
             )
             .replace("##DECLS##", "\n".join(decls))
-            .replace("##CALLS##", "\n".join(calls))
+            .replace("##BEGIN_CALLS##", "\n".join(begin_calls))
+            .replace("##FINISH_CALLS##", "\n".join(finish_calls))
         )
 
         with open(join(outdir, "rpygen_wrapper.hpp"), "w") as fp:
